<?php

namespace Drupal\image_sizes\Command;

use Drupal\image\Entity\ImageStyle;
use Drupal\Console\Core\Command\Command;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\Element\MachineName;
use Symfony\Component\Console\Input\InputOption;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Drupal\image_sizes\Entity\ImageSizesPresetEntity;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ConfirmationQuestion;

/**
 * Class GenerateImageSizeCommand.
 *
 * Drupal\Console\Annotations\DrupalCommand (
 *     extension="image_sizes",
 *     extensionType="module"
 * )
 */
class GenerateImageSizeCommand extends Command {

  protected $ratio = FALSE;

  protected $useFocalPoint = FALSE;

  /**
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var Drupal\Core\Extension\ModuleHandle
   */
  protected $moduleHandler;

  /**
   * DeleteCommand constructor.
   *
   * @param EntityTypeManagerInterface    $entityTypeManager
   */
  public function __construct(
      EntityTypeManagerInterface $entityTypeManager,
      ModuleHandlerInterface $module_handler
  ) {
      $this->entityTypeManager = $entityTypeManager;
      $this->moduleHandler = $module_handler;
      parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  protected function configure() {
    $this
      ->setName('image_sizes:generate')
      ->setDescription($this->trans('commands.image_sizes.generate.description'))
      ->addArgument(
        'name',
        InputArgument::REQUIRED,
        $this->trans('commands.image_sizes.generate.arguments.name')
      )
      ->addArgument(
        'min',
        InputArgument::REQUIRED,
        $this->trans('commands.image_sizes.generate.arguments.min')
      )
      ->addArgument(
        'max',
        InputArgument::REQUIRED,
        $this->trans('commands.image_sizes.generate.arguments.max')
      )->addArgument(
        'step',
        InputArgument::REQUIRED,
        $this->trans('commands.image_sizes.generate.arguments.step')
      )->addOption(
        'ratio',
        null,
        InputOption::VALUE_OPTIONAL,
        $this->trans('commands.image_sizes.generate.options.step'),
        false
      );

  }

  /**
   * {@inheritdoc}
   */
  protected function interact(InputInterface $input, OutputInterface $output)
  {
    if (!$input->getOption("ratio") || !$this->moduleHandler->moduleExists('focal_point')) {
      return;
    }
    if ($this->getIo()->askQuestion(new ConfirmationQuestion($this->trans('commands.image_sizes.generate.use_focal_point'), true))) {
      $this->useFocalPoint = TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function initialize(InputInterface $input, OutputInterface $output) {
    parent::initialize($input, $output);
    $this->getIo()->info('initialize');
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output) {
    $ratio = $input->getOption('ratio');
    $name = $input->getArgument('name');
    $min = $input->getArgument('min');
    $max = $input->getArgument('max');
    $step = $input->getArgument('step');
    $ratios = ['x' => 0, 'y' => 0];
    $preloadStyle = FALSE;

    $style = ImageStyle::load($name);
    if ($style) {
      $this->getIo()->warning('Already exists');
      return;
    }

    if ($ratio != false) {
      $reg = '/^(\d+)x(\d+)$/';
      $matches = [];
      preg_match_all($reg, $ratio, $matches, PREG_SET_ORDER, 0);
      if (empty($matches)) {
        $this->getIo()->error('Not correct ratio');
        return;
      }
      $ratio = TRUE;
      $ratios = ['x' => $matches[0][1], 'y' => $matches[0][2]];
    }

    $preset = ImageSizesPresetEntity::create([
      'label' => "$name",
      'id' => self::machineName($name),
    ]);

    for ($i = $min; $i < $max; $i = $i + $step ) {
      $style_name = $name . '_' . $i;
      $x = $i;

      $style = ImageStyle::create([
        'name' => $style_name,
        'label' => 'Image Style ${x}'
      ]);
      $configuration = array(
        'uuid' => NULL,
        'id' => 'image_scale',
        'weight' => 0,
        'data' => array(
          'width' => $x,
        ),
      );
      if ($ratio == TRUE) {
        $m = floor($i / $ratios['x']);
        $x = $ratios['x'] * $m;
        $y = $ratios['y'] * $m;
        $style = ImageStyle::create([
          'name' => $style_name,
          'label' => "Image style ${x}x${y}"]
        );
        $configuration = array(
          'uuid' => NULL,
          'id' => ($this->useFocalPoint) ? 'focal_point_scale_and_crop' : 'image_scale_and_crop',
          'weight' => 0,
          'data' => array(
            'width' => $x,
            'height' => $y,
          ),
        );
      }

      $effect = \Drupal::service('plugin.manager.image.effect')->createInstance($configuration['id'], $configuration);
      $style->addImageEffect($effect->getConfiguration());
      $style->save();
      $this->getIo()->info("Generate Style ${style_name}");
      $preset->addStyle($style->id());
      if ($preloadStyle == FALSE) {
        $preset->setPreloadStyle($style->id());
        $preloadStyle = TRUE;
      }
    }
    $preset->setFallbackStyle($style->id());
    $preset->save();

    $this->getIo()->info('execute');
    $this->getIo()->info($this->trans('commands.image_sizes.generate.messages.success', ['@style', $style_name]));
  }

  public static function machineName($value) {
    $new_value = \Drupal::service('transliteration')->transliterate($value, LanguageInterface::LANGCODE_DEFAULT, '_');
    $new_value = strtolower($new_value);
    $new_value = preg_replace('/[^a-z0-9_]+/', '_', $new_value);
    return preg_replace('/_+/', '_', $new_value);
  }

}
