<?php

namespace Drupal\image_sizes\Plugin\Field\FieldFormatter;

use Drupal\image_sizes\ImageSizesFormatterTrait;
use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Component\Serialization\Json;

/**
 * Plugin implementation of the 'image_sizes_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "image_sizes_formatter",
 *   label = @Translation("Image sizes formatter"),
 *   field_types = {
 *     "image",
 *     "entity_reference"
 *   }
 * )
 */
class ImageSizesFormatter extends FormatterBase {

  use ImageSizesFormatterTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
      'image_style_placeholder' => FALSE,
      'image_styles' => [],
      'load_invisible' => FALSE,
      'fallback_style' => 'original',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      // Implement settings form.
      'load_invisible' => [
        '#type' => 'checkbox',
        '#default_value' => $this->getSetting('load_invisible'),
        '#title' => $this->t('Load this image even is not visible'),
      ],
      'image_style_placeholder' => [
        '#type' => 'select',
        '#required' => TRUE,
        '#title' => $this->t('Placeholder image style.'),
        '#options' => image_style_options(),
        '#default_value' => $this->getSetting('image_style_placeholder'),
      ],
      'image_styles' => [
        '#type' => 'select',
        '#multiple' => TRUE,
        '#required' => TRUE,
        '#options' => $this->getOptions(),
        '#title' => $this->t('Choose image styles for each width.'),
        '#default_value' => $this->getSetting('image_styles'),
      ],
      'fallback_style' => [
        '#type' => 'select',
        '#required' => TRUE,
        '#title' => $this->t('Fallback image if no style is selectable.'),
        '#options' => array_merge(
          ['original' => t('Original')],
          image_style_options(FALSE)
        ),
        '#default_value' => $this->getSetting('fallback_style'),
      ],
    ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    return $summary;
  }

  /**
   * Get all options for select a style.
   */
  protected function getOptions() {
    $image_styles = \Drupal::entityQuery('image_style')
      ->execute();
    $styles = [];
    foreach ($image_styles as $style_name) {
      $style = ImageStyle::load($style_name);
      $effects = $style->getEffects();
      foreach ($effects as $effect) {
        if (is_a($effect, 'Drupal\image\Plugin\ImageEffect\ResizeImageEffect')) {
          $width = $effect->getConfiguration()['data']['width'];
          if ($width !== NULL && $width > 0) {
            $styles[$style_name] = $style->label() . " ($width)";
          }
        }
      }
    }
    return $styles;
  }

  /**
   * Get all uris for selected styles.
   */
  protected function getStyles($uri) {
    $image_styles = \Drupal::entityQuery('image_style')
      ->condition('name', $this->getSetting('image_styles'), 'IN')
      ->execute();
    $styles = [];
    foreach ($image_styles as $style_name) {
      $style = ImageStyle::load($style_name);
      $effects = $style->getEffects();
      foreach ($effects as $effect) {
        if (is_a($effect, 'Drupal\image\Plugin\ImageEffect\ResizeImageEffect')) {
          $width = $effect->getConfiguration()['data']['width'];
          if ($width !== NULL && $width > 0) {
            $styles[$width] = $style->buildUrl($uri);
          }
        }
      }
    }
    ksort($styles);
    return $styles;
  }

  /**
   * Get uri for fallback style or original image if no fallback is assigned.
   */
  protected function getFallBackStyle($uri) {
    if ($this->getSetting('fallback_style') == 'original') {
      return file_create_url($uri);
    }

    $style = ImageStyle::load($this->getSetting('fallback_style'));
    if (!$style) {
      return file_create_url($uri);
    }

    return $style->buildUrl($uri);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $styles[] = $this->getSetting('image_style_placeholder');
    $styles += $this->getSetting('image_styles');
    $styles[] = $this->getSetting('fallback_style');

    foreach ($styles as $style_id) {
      /** @var \Drupal\image\ImageStyleInterface $style */
      if ($style_id && $style = ImageStyle::load($style_id)) {
        // If this formatter uses a valid image style to display the image, add
        // the image style configuration entity as dependency of this formatter.
        $dependencies[$style->getConfigDependencyKey()][] = $style->getConfigDependencyName();
      }
    }
    return $dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      $elements[$delta] = $this->viewValue($item);
    }
    $elements['#attached']['library'] = ['image_sizes/core'];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  protected function viewValue(FieldItemInterface $item) {
    $uri = $this->getFileUri($item);
    if (!$uri) {
      return [];
    }
    $render = [
      '#theme' => 'image_style',
      '#style_name' => $this->getSetting('image_style_placeholder'),
      '#uri' => $uri,
      '#attributes' => [
        'data-src' => Json::encode($this->getStyles($uri)),
        'class' => [
          'image-sizes',
          'pre-load',
        ],
        'data-src-fallback' => $this->getFallBackStyle($uri),
      ],
    ];
    if ($this->getSetting('load_invisible')) {
      $render['#attributes']['class'][] = 'load-always';
    }
    if (isset($item->title)) {
      $render['#attributes']['title'] = $item->title;
    }
    if (isset($item->alt)) {
      $render['#attributes']['alt'] = $item->alt;
    }

    return $render;
  }

}
