<?php

namespace Drupal\image_sizes\Plugin\ImageEffect;

use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Image\ImageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\ConfigurableImageEffectBase;

/**
 * Crops an image resource.
 *
 * @ImageEffect(
 *   id = "image_sizes_insert",
 *   label = @Translation("Insert image style"),
 *   description = @Translation("Resizing will make images an exact set of dimensions. This may cause images to be stretched or shrunk disproportionately.")
 * )
 */
class InsertStyleEffect extends ConfigurableImageEffectBase {

  /**
   * {@inheritdoc}
   */
  public function applyEffect(ImageInterface $image) {
    $style = $this->failSafeGetImageStyle($this->configuration['style']);
    if ($style === FALSE) {
      return FALSE;
    }
    foreach ($style->getEffects() as $effect) {
      if ($effect->applyEffect($image) !== TRUE) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $summary = [
      '#theme' => 'image_sizes_insert_summary',
      '#data' => $this->configuration,
    ];
    $summary += parent::getSummary();

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'style' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $image_styles = [];
    if ($style = $this->failSafeGetImageStyle($this->configuration['style'])) {
      $image_styles[] = $style->getConfigDependencyName();
    }
    return ['config' => $image_styles];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $options = image_style_options();
    $style = $form_state->getBuildInfo()['args'][0];
    if ($style) {
      unset($options[$style->id()]);
    }
    $form['style'] = [
      '#type' => 'select',
      '#title' => t('Image style'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $this->configuration['style'],
      '#description' => t('The part of the image that will be retained during the crop.'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['style'] = $form_state->getValue('style');
  }

  /**
   * {@inheritdoc}
   */
  public function transformDimensions(array &$dimensions, $uri) {
    // The new image will have the exact dimensions defined for the effect.
    if ($style = $this->failSafeGetImageStyle($this->configuration['style'])) {
      $style->transformDimensions($dimensions, $uri);
    }
  }

  /**
   * Helper function for loading a style.
   *
   * @param string $image_style_name
   *   Style name to load.
   *
   * @return Drupal\image\Entity\ImageStyle|null
   *   Image style to load.
   */
  protected function failSafeGetImageStyle($image_style_name) {
    if ($image_style_name === NULL) {
      return NULL;
    }
    $image_style = ImageStyle::load($image_style_name);
    if ($image_style === NULL) {
      // Required style has gone missing?
      $this->logger->error("Cannot find image style '%style_name' to execute an 'insert image style' effect.", ['%style_name' => $image_style_name]);
      return FALSE;
    }
    return $image_style;
  }

}
